@@ -5,9 +5,9 @@ use strict;
 use warnings;
 
 my $Indicators = __PACKAGE__->INDICATORS;
+my $ReBackbone = qr|^---[ ]The[ ]header[ ]of[ ]the[ ]original[ ]message[ ]is[ ]following[.][ ]---|m;
 my $StartingOf = {
     'message' => ['This message was created automatically by mail delivery software'],
-    'rfc822'  => ['--- The header of the original message is following'],
     'error'   => ['For the following reason:'],
 };
 my $MessagesOf = { 'mesgtoobig' => ['Mail size limit exceeded'] };
@@ -35,80 +35,78 @@ sub make {
     return undef unless $mhead->{'subject'} eq 'Mail delivery failed: returning message to sender';
 
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my $rfc822part = '';    # (String) message/rfc822-headers part
-    my $rfc822list = [];    # (Array) Each line in message/rfc822 part string
-    my $blanklines = 0;     # (Integer) The number of blank lines
+    my $emailsteak = Sisimai::RFC5322->fillet($mbody, $ReBackbone);
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $v = undef;
 
-    for my $e ( split("\n", $$mbody) ) {
-        # Read each line between the start of the message and the start of rfc822 part.
+    for my $e ( split("\n", $emailsteak->[0]) ) {
+        # Read error messages and delivery status lines from the head of the email
+        # to the previous line of the beginning of the original message.
         unless( $readcursor ) {
-            # Beginning of the bounce message or delivery status part
-            if( index($e, $StartingOf->{'message'}->[0]) == 0 ) {
-                $readcursor |= $Indicators->{'deliverystatus'};
-                next;
-            }
+            # Beginning of the bounce message or message/delivery-status part
+            $readcursor |= $Indicators->{'deliverystatus'} if index($e, $StartingOf->{'message'}->[0]) == 0;
+            next;
         }
+        next unless $readcursor & $Indicators->{'deliverystatus'};
+        next unless length $e;
 
-        unless( $readcursor & $Indicators->{'message-rfc822'} ) {
-            # Beginning of the original message part
-            if( index($e, $StartingOf->{'rfc822'}->[0]) == 0 ) {
-                $readcursor |= $Indicators->{'message-rfc822'};
-                next;
-            }
-        }
+        # The following address failed:
+        #
+        # general@example.eu
+        #
+        # For the following reason:
+        #
+        # Mail size limit exceeded. For explanation visit
+        # http://postmaster.1and1.com/en/error-messages?ip=%1s
+        $v = $dscontents->[-1];
 
-        if( $readcursor & $Indicators->{'message-rfc822'} ) {
-            # Inside of the original message part
-            unless( length $e ) {
-                last if ++$blanklines > 1;
-                next;
+        if( $e =~ /\A([^ ]+[@][^ ]+)\z/ ) {
+            # general@example.eu
+            if( $v->{'recipient'} ) {
+                # There are multiple recipient addresses in the message body.
+                push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+                $v = $dscontents->[-1];
             }
-            push @$rfc822list, $e;
+            $v->{'recipient'} = $1;
+            $recipients++;
+
+        } elsif( index($e, $StartingOf->{'error'}->[0]) == 0 ) {
+            # For the following reason:
+            $v->{'diagnosis'} = $e;
 
         } else {
-            # Error message part
-            next unless $readcursor & $Indicators->{'deliverystatus'};
-            next unless length $e;
-
-            # The following address failed:
-            #
-            # general@example.eu
-            #
-            # For the following reason:
-            #
-            # Mail size limit exceeded. For explanation visit
-            # http://postmaster.1and1.com/en/error-messages?ip=%1s
-            $v = $dscontents->[-1];
-
-            if( $e =~ /\A([^ ]+[@][^ ]+)\z/ ) {
-                # general@example.eu
-                if( $v->{'recipient'} ) {
-                    # There are multiple recipient addresses in the message body.
-                    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-                    $v = $dscontents->[-1];
-                }
-                $v->{'recipient'} = $1;
-                $recipients++;
-
-            } elsif( index($e, $StartingOf->{'error'}->[0]) == 0 ) {
-                # For the following reason:
-                $v->{'diagnosis'} = $e;
+            if( length $v->{'diagnosis'} ) {
+                # Get error message and append the error message strings
+                $v->{'diagnosis'} .= ' '.$e;
 
             } else {
-                # Get error message and append the error message strings
-                $v->{'diagnosis'} .= ' '.$e if $v->{'diagnosis'};
+                # OR the following format:
+                #   neko@example.fr:
+                #   SMTP error from remote server for TEXT command, host: ...
+                $v->{'alterrors'} .= ' '.$e;
             }
-        } # End of error message part
+        }
     }
     return undef unless $recipients;
 
     for my $e ( @$dscontents ) {
-        $e->{'agent'}     =  __PACKAGE__->smtpagent;
-        $e->{'diagnosis'} =~ s/\A$StartingOf->{'error'}->[0]//g;
-        $e->{'diagnosis'} =  Sisimai::String->sweep($e->{'diagnosis'});
+        $e->{'agent'}       =  __PACKAGE__->smtpagent;
+        $e->{'diagnosis'} ||= $e->{'alterrors'} || '';
+
+        if( $e->{'diagnosis'} =~ /host:[ ]+(.+?)[ ]+.+[ ]+reason:.+/ ) {
+            # SMTP error from remote server for TEXT command,
+            #   host: smtp-in.orange.fr (193.252.22.65)
+            #   reason: 550 5.2.0 Mail rejete. Mail rejected. ofr_506 [506]
+            $e->{'rhost'}   = $1;
+            $e->{'command'} = 'DATA' if $e->{'diagnosis'} =~ /for TEXT command/;
+            $e->{'spec'}    = 'SMTP' if $e->{'diagnosis'} =~ /SMTP error/;
+            $e->{'status'}  = Sisimai::SMTP::Status->find($e->{'diagnosis'});
+        } else {
+            # For the following reason:
+            $e->{'diagnosis'} =~ s/\A$StartingOf->{'error'}->[0]//g;
+        }
+        $e->{'diagnosis'} = Sisimai::String->sweep($e->{'diagnosis'});
 
         SESSION: for my $r ( keys %$MessagesOf ) {
             # Verify each regular expression of session errors
@@ -117,8 +115,7 @@ sub make {
             last;
         }
     }
-    $rfc822part = Sisimai::RFC5322->weedout($rfc822list);
-    return { 'ds' => $dscontents, 'rfc822' => $$rfc822part };
+    return { 'ds' => $dscontents, 'rfc822' => $emailsteak->[1] };
 }
 
 1;
@@ -164,7 +161,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2019 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2020 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
