@@ -5,9 +5,9 @@ use strict;
 use warnings;
 
 my $Indicators = __PACKAGE__->INDICATORS;
+my $ReBackbone = qr|^Content-Type:[ ]message/rfc822|m;
 my $StartingOf = {
     'message' => ['This is a MIME encoded message'],
-    'rfc822'  => ['Content-Type: message/rfc822'],
     'error'   => ['Delivery failed for the following reason:'],
 };
 
@@ -33,78 +33,55 @@ sub make {
     return undef unless grep { rindex($_, '(MAILFOUNDRY) id') > -1 } @{ $mhead->{'received'} };
 
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my $rfc822part = '';    # (String) message/rfc822-headers part
-    my $rfc822list = [];    # (Array) Each line in message/rfc822 part string
-    my $blanklines = 0;     # (Integer) The number of blank lines
+    my $emailsteak = Sisimai::RFC5322->fillet($mbody, $ReBackbone);
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $v = undef;
 
-    for my $e ( split("\n", $$mbody) ) {
-        # Read each line between the start of the message and the start of rfc822 part.
+    for my $e ( split("\n", $emailsteak->[0]) ) {
+        # Read error messages and delivery status lines from the head of the email
+        # to the previous line of the beginning of the original message.
         unless( $readcursor ) {
-            # Beginning of the bounce message or delivery status part
-            if( $e eq $StartingOf->{'message'}->[0] ) {
-                $readcursor |= $Indicators->{'deliverystatus'};
-                next;
-            }
+            # Beginning of the bounce message or message/delivery-status part
+            $readcursor |= $Indicators->{'deliverystatus'} if index($e, $StartingOf->{'message'}->[0]) == 0;
+            next;
         }
+        next unless $readcursor & $Indicators->{'deliverystatus'};
+        next unless length $e;
 
-        unless( $readcursor & $Indicators->{'message-rfc822'} ) {
-            # Beginning of the original message part
-            if( $e eq $StartingOf->{'rfc822'}->[0] ) {
-                $readcursor |= $Indicators->{'message-rfc822'};
-                next;
-            }
-        }
+        # Unable to deliver message to: <kijitora@example.org>
+        # Delivery failed for the following reason:
+        # Server mx22.example.org[192.0.2.222] failed with: 550 <kijitora@example.org> No such user here
+        #
+        # This has been a permanent failure.  No further delivery attempts will be made.
+        $v = $dscontents->[-1];
 
-        if( $readcursor & $Indicators->{'message-rfc822'} ) {
-            # Inside of the original message part
-            unless( length $e ) {
-                last if ++$blanklines > 1;
-                next;
+        if( $e =~ /\AUnable to deliver message to: [<]([^ ]+[@][^ ]+)[>]\z/ ) {
+            # Unable to deliver message to: <kijitora@example.org>
+            if( $v->{'recipient'} ) {
+                # There are multiple recipient addresses in the message body.
+                push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+                $v = $dscontents->[-1];
             }
-            push @$rfc822list, $e;
+            $v->{'recipient'} = $1;
+            $recipients++;
 
         } else {
-            # Error message part
-            next unless $readcursor & $Indicators->{'deliverystatus'};
-            next unless length $e;
-
-            # Unable to deliver message to: <kijitora@example.org>
-            # Delivery failed for the following reason:
-            # Server mx22.example.org[192.0.2.222] failed with: 550 <kijitora@example.org> No such user here
-            #
-            # This has been a permanent failure.  No further delivery attempts will be made.
-            $v = $dscontents->[-1];
-
-            if( $e =~ /\AUnable to deliver message to: [<]([^ ]+[@][^ ]+)[>]\z/ ) {
-                # Unable to deliver message to: <kijitora@example.org>
-                if( $v->{'recipient'} ) {
-                    # There are multiple recipient addresses in the message body.
-                    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-                    $v = $dscontents->[-1];
-                }
-                $v->{'recipient'} = $1;
-                $recipients++;
+            # Error message
+            if( $e eq $StartingOf->{'error'}->[0] ) {
+                # Delivery failed for the following reason:
+                $v->{'diagnosis'} = $e;
 
             } else {
-                # Error message
-                if( $e eq $StartingOf->{'error'}->[0] ) {
-                    # Delivery failed for the following reason:
-                    $v->{'diagnosis'} = $e;
+                # Detect error message
+                next unless length $e;
+                next unless $v->{'diagnosis'};
+                next if index($e, '-') == 0;
 
-                } else {
-                    # Detect error message
-                    next unless length $e;
-                    next unless $v->{'diagnosis'};
-                    next if index($e, '-') == 0;
-
-                    # Server mx22.example.org[192.0.2.222] failed with: 550 <kijitora@example.org> No such user here
-                    $v->{'diagnosis'} .= ' '.$e;
-                }
+                # Server mx22.example.org[192.0.2.222] failed with: 550 <kijitora@example.org> No such user here
+                $v->{'diagnosis'} .= ' '.$e;
             }
-        } # End of error message part
+        }
     }
     return undef unless $recipients;
 
@@ -113,9 +90,7 @@ sub make {
         $e->{'diagnosis'} = Sisimai::String->sweep($e->{'diagnosis'});
         $e->{'agent'}  = __PACKAGE__->smtpagent;
     }
-
-    $rfc822part = Sisimai::RFC5322->weedout($rfc822list);
-    return { 'ds' => $dscontents, 'rfc822' => $$rfc822part };
+    return { 'ds' => $dscontents, 'rfc822' => $emailsteak->[1] };
 }
 
 1;
@@ -161,7 +136,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2019 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2020 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
